#include <netdef.h>
#include <proto.h>
#ifdef DEBUG
#include <util.h>
#include <errno.h>
#include <stdio.h>
#endif

#include <sys/socket.h>
#include <netinet/in.h>		/* sockaddr_in, htons, in_addr */
#include <netinet/in_systm.h>	/* misc crud that netinet/ip.h references */
#include <netinet/ip.h>		/* IPOPT_LSRR, header stuff */
#include <netinet/tcp.h>

int udp_multi_accept(netnode n1, netnode n2, int flags, netnode *n){
  return n1.fd;
}

int udp_multi_send(netnode n1, netnode n2, int flags, char *data, int size){
  int sock = n1.fd;  /* n1 should always hold the proper file descriptor */
  int len = size, done = 0, tmp = 0;

#ifdef DEBUG
  char *sbuf = (char *)malloc(sizeof(data) + 255);
  sprintf(sbuf, "Sending: %s\nSocket #:%d", data, sock);
  __util_debug_print(sbuf);
#endif 
  do{
    
    if((tmp = sendto(sock, data+done, len, 0, (struct sockaddr *)&n2.in, sizeof(struct sockaddr))) < 0){
      /* errors */
      return -1;
    }
    
  }while( (done += tmp) < len) ;

#ifdef DEBUG
  __util_debug_print("data sent");
#endif
  
  return done;
}

int udp_multi_recv(netnode n1, netnode n2, int flags, char *data, int size){
  int i = 0;
#ifdef DEBUG
  __util_debug_print("Receiving data...");
#endif

  i = recvfrom(n1.fd, data, size, 0, (struct sockaddr *)&n2.in, sizeof(struct sockaddr));

  return i;
}

int udp_multi_conn(netnode n1, netnode n2, int flags){
  int sd = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
  int loop = 0;
  struct ip_mreq ipm;

#ifdef DEBUG
  char sbuf[255];
  __util_debug_print("We got a socket...");
#endif

  if(n1.junk != 0){
    if(bind(sd, (struct sockaddr *)&n1.in, sizeof(struct sockaddr)) < 0){
      return -1;
    }
  }
  n1.junk = 0;

  setsockopt(sd, IPPROTO_IP, IP_MULTICAST_LOOP, &loop, sizeof(loop));
  ipm.imr_multiaddr = n2.in.sin_addr;
  ipm.imr_interface.s_addr = INADDR_ANY;
  /* we're joined to the multicast address */
  setsockopt(sd, IPPROTO_IP, IP_ADD_MEMBERSHIP, &ipm, sizeof(struct ip_mreq));


#ifdef DEBUG
  sprintf(sbuf, "Socket#: %d", sd);
  __util_debug_print(sbuf);
#endif

  return sd;
}

int udp_multi_disconn(netnode n1, netnode n2){
  return close(n1.fd);
}

protocol udpmulticast = {
  "udp-multicast",
  0,
  udp_multi_accept,
  udp_multi_send,
  udp_multi_recv,
  udp_multi_conn,
  udp_multi_disconn  
};

