/**
 * This is just a Proof of Concept protocol to show that libnpconn can 
 * communicate with serial devices as well as with regular sockets.
 * It's useless for the accept() and netnode() junk, but we can get around
 * that stuff. Remember, the netnode structure contains a char* indicating
 * the text version of the host address
 *
 * We simply ignore the second netnode (the remote portion) because we don't
 * need it. This means, 1) we can now communicate with serial devices within
 * the framework (for localized I/O) and 2) we can show something that npconn
 * might not have originally been devised to do, but is flexible enough to 
 * handle. It doesn't violate what npconn was originally designed for. npconn
 * was always about I/O connection channels.
 * --aaron
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <termios.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <netdef.h>
#include <proto.h>
#include <proto-serial.h>

#ifdef DEBUG
#include <util.h>
#include <errno.h>
#endif

protocol serialcomm = {
  "serial-comm",
  0,
  nullaccept,          /* I've realized that someone may have some generic
			  routine handler to auto call accept(). this is NOT
			  the right thing to do, but we'll accommodate it */
  serialsend,
  serialrecv,
  serialcon,
  serialdiscon
};

int nullaccept(netnode n1, netnode n, int flags, netnode *g){
#ifdef DEBUG
  __util_debug_print("ERROR: accept() on a serial connection is wrong.");
#endif
  return -1;
}

int serialsend(netnode n1, netnode n2, int flags, char *data, int size){
  /* we should do something with flags passed in, but until I really
     grasp termios better, we'll just do this */
  return write(n1.fd, data, size);
}

int serialrecv(netnode n1, netnode n2, int flags, char *data, int size){
  /* again, the flags thing...they're only used in the beginning, sadly */
  return read(n1.fd, data, size);
}

/**
 * this function is BIG!! It's the first function that really only
 * gets effective when you use flags. the flags are:
 * BAUD300, BAUD1200, BAUD9600, BAUD19200, BAUD38400, S1, S2, A7, A8, NONE, 
 * EVEN, ODD the BAUD<number> are indicative of the speed. S1 and S2 are which
 * stopbit setting you are using. The rest are just parity. Usually, the 
 * default is 9600,8n1 so that would look like:
 *   BAUD9600 | S1 | A8 | NONE
 * This has been mapped to default flags.
 */
int serialcon(netnode n1, netnode n2, int flags){
  int fd, flagdec = 0;
  struct termios tios;

  if(n1.host == NULL) return -2;

  fd = open(n1.host, O_RDWR | O_NOCTTY ); /* make sure not to close controlling
					     tty with CTRL-C...etc */
  if (fd <0) return -1;
  bzero(&tios, sizeof(tios));             /* clear the terminal */

  tcgetattr(fd, &tios); /* this is to get the initial terminal values */

  if(flags & BAUD300){
    flagdec |= B300;
  } else if(flags & BAUD1200){
    flagdec |= B1200;
  } else if(flags & BAUD19200){
    flagdec |= B19200;
  } else if(flags & BAUD38400){
    flagdec |= B38400;
  } else
    flagdec |= B9600;

  if(flags & A7)
    flagdec |= CS7;
  else
    flagdec |= CS8;

  if(flags & SUPER_MYSTERY)
    flagdec |= CRTSCTS;
  
  flagdec |= CLOCAL | CREAD;

  tios.c_cflag |= flagdec;

  tios.c_iflag = IGNPAR | ICRNL;

  tios.c_oflag = IGNPAR | ICRNL;
  
  tios.c_lflag = ICANON;

  /* initialize the control characters */
  
  tios.c_cc[VINTR]    = 0;     /* Ctrl-c */ 
  tios.c_cc[VQUIT]    = 0;     /* Ctrl-\ */
  tios.c_cc[VERASE]   = 0;     /* del */
  tios.c_cc[VKILL]    = 0;     /* @ */
  tios.c_cc[VEOF]     = 4;     /* Ctrl-d */
  tios.c_cc[VTIME]    = 0;     /* inter-character timer unused */
  tios.c_cc[VMIN]     = 1;     /* blocking read until 1 character arrives */
  tios.c_cc[VSWTC]    = 0;     /* '\0' */
  tios.c_cc[VSTART]   = 0;     /* Ctrl-q */ 
  tios.c_cc[VSTOP]    = 0;     /* Ctrl-s */
  tios.c_cc[VSUSP]    = 0;     /* Ctrl-z */
  tios.c_cc[VEOL]     = 0;     /* '\0' */
  tios.c_cc[VREPRINT] = 0;     /* Ctrl-r */
  tios.c_cc[VDISCARD] = 0;     /* Ctrl-u */
  tios.c_cc[VWERASE]  = 0;     /* Ctrl-w */
  tios.c_cc[VLNEXT]   = 0;     /* Ctrl-v */
  tios.c_cc[VEOL2]    = 0;     /* '\0' */
  
  tcflush(fd, TCIFLUSH); /* flush the line */
  tcsetattr(fd, TCSANOW, &tios); /* activate the new settings */

  /* we can now return */
  return fd;
}

int serialdiscon(netnode n1, netnode n2){
  return close(n1.fd);
}
